/* tests whether hard disk writes in-order */
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <linux/fs.h>
#include <stdlib.h>
#include "blkseek.h"

int main(int argc, char *argv[])
{
  char *device;
  Int magic, skip;
  int fd;
  Int blockhead[2];
  Int i,past;
  Int commited;

  if (argc != 3) {
    fprintf(stderr,"Usage: %s <device> <skip>\n", argv[0]);
    exit(1);
  }
  device=argv[1];
  skip=atoi(argv[2])+1;
  fd=open(device, O_SYNC|O_RDONLY);
  if (fd<0) {
    perror("open");
    exit(1);
  }
  if (!blkseek(fd,0)) {
    perror("seek(0)");
    exit(1);
  }
  if (read(fd,(char *)blockhead,2*sizeof(Int))!=2*sizeof(Int)) {
    perror("read(0)");
    exit(1);
  }
  commited=blockhead[0];
  magic=blockhead[1];
  printf("last committed: %ld; magic: %ld\n",commited,magic);
  for (i = commited-16*skip; i<commited+16*skip; i += skip) {
    if (!blkseek(fd,i)) {
      perror("seek(i)");
      exit(1);
    }
    if (read(fd,(char *)blockhead,2*sizeof(Int))!=2*sizeof(Int)) {
      perror("read(0)");
      exit(1);
    }
    if (i==commited+skip)
      printf("**** everything before should have correct magic ******\n");
    if (i==commited+2*skip)
      printf("**** nothing below should have correct magic ******\n");
    printf("blockid: %ld; magic: %ld\n",blockhead[0],blockhead[1]);
  }
  if (blockhead[1] != magic) {
    exit(1);
  }
  for (past=14;;i+=skip,past++) {
    if (!blkseek(fd,i)) {
      perror("seek(i)");
      exit(1);
    }
    if (read(fd,(char *)blockhead,2*sizeof(Int))!=2*sizeof(Int)) {
      perror("read(0)");
      exit(1);
    }
    if (blockhead[1] != magic) {
      printf("\nwrote %ld blocks out-of-order\n",past);
      exit(0);
    } else {
      printf("\r... blockid: %ld", i);
    }
  }   
  exit(0);
}
